import { __awaiter, __decorate, __param } from "tslib";
import { Auth, Connect } from '@genesislcap/foundation-comms';
import { DI } from '@microsoft/fast-foundation';
import { DEBOUNCED_SEARCH_TIME } from '../main/main.config';
import { NotifyResourceNames } from '../main/main.types';
import { logger, transformMessageToNotify } from '../utils';
let DefaultFoundationInboxService = class DefaultFoundationInboxService {
    constructor(connect, auth) {
        this.connect = connect;
        this.auth = auth;
        this.resourceName = NotifyResourceNames.ALL_RECORDS;
        this.subscriptions = [];
        this.store = {
            items: [],
        };
    }
    getStream() {
        return this.connect.stream(this.resourceName, (result) => {
            this.sourceRef = result.SOURCE_REF;
            if (!(result === null || result === void 0 ? void 0 : result.ROW)) {
                this.store.items = [];
                return;
            }
            if (result.SEQUENCE_ID === 1) {
                const newNotifies = [];
                result.ROW.forEach((row) => newNotifies.push(transformMessageToNotify(row)));
                this.store.items = newNotifies;
            }
            else {
                const details = result === null || result === void 0 ? void 0 : result.ROW[0];
                if ((details === null || details === void 0 ? void 0 : details.DETAILS.OPERATION) === 'DELETE') {
                    const notifyItemIndex = this.store.items.findIndex(({ rowRef }) => rowRef === details.DETAILS.ROW_REF);
                    if (notifyItemIndex > -1) {
                        this.store.items.splice(notifyItemIndex, 1);
                    }
                }
                if ((details === null || details === void 0 ? void 0 : details.DETAILS.OPERATION) === 'INSERT') {
                    this.store.items.push(transformMessageToNotify(details));
                }
            }
            this.subscriptions.forEach((callback) => callback({ store: this.store }));
        }, (response) => logger.error('Error: Could not load notify data.' + response));
    }
    subscribe(callback) {
        this.subscriptions.push(callback);
        if (this.stream) {
            callback({ store: this.store });
        }
        else {
            this.stream = this.getStream();
            this.stream.subscribe();
        }
    }
    unsubscribe(callback) {
        const index = this.subscriptions.indexOf(callback);
        if (index >= 0) {
            this.subscriptions.splice(index, 1);
        }
        this.dataLogoff();
    }
    dataLogoff() {
        if (this.sourceRef && this.auth.isLoggedIn) {
            this.connect.dataLogoff(this.sourceRef);
        }
        this.stream = undefined;
        this.store = { items: [] };
        this.sourceRef = undefined;
    }
    filter(filterTypes, payload) {
        const criteriaByFilterType = {
            search: `(Expr.containsIgnoreCase(MESSAGE, '${payload['search']}') || Expr.containsIgnoreCase(HEADER, '${payload['search']}'))`,
            severity: `NOTIFY_SEVERITY == '${payload['severity']}'`,
        };
        const CRITERIA_MATCH = filterTypes
            .map((filterType) => criteriaByFilterType[filterType])
            .join(' && ');
        return new Promise((resolve) => {
            if (this.debounced) {
                clearTimeout(this.debounced);
                this.debounced = null;
            }
            this.debounced = setTimeout(() => __awaiter(this, void 0, void 0, function* () {
                const { ROW: rows } = yield this.connect.snapshot(this.resourceName, { CRITERIA_MATCH });
                let newNotifies = [];
                if (rows) {
                    newNotifies = rows.map((row) => transformMessageToNotify(row));
                }
                return resolve(newNotifies);
            }), DEBOUNCED_SEARCH_TIME);
        });
    }
};
DefaultFoundationInboxService = __decorate([
    __param(0, Connect),
    __param(1, Auth)
], DefaultFoundationInboxService);
export const FoundationInboxService = DI.createInterface((x) => x.singleton(DefaultFoundationInboxService));
